﻿--[[
	Project: HideParty
	Version: 2.1
	Website: http://wow.curseforge.com/addons/hideparty/
	Author: Erfolg

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
]] 

HideParty = LibStub("AceAddon-3.0"):NewAddon("HideParty", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("HideParty")
local db = nil
local c_400 = select(4,GetBuildInfo()) >= 40000
local c_410 = select(4,GetBuildInfo()) >= 40100

raidHidden = nil
partyHidden = nil
playerHidden = nil
targetHidden = nil

local options = {
name = "HideParty",
handler = HideParty,
type = "group",
	
	args = {
		
		descrp = {
		type = "description",
		name = L["addon_desc"],
		order = 0,	  
		},

		sep = {
		type = "header",
		name = "",
		order = 1,
		},

		targetEnabled = {
		type = "toggle",
		name = L["HideTarget"],
		desc = L["HideTargetDesc"],
		get = function() return db.targetEnabled end,
		set = function(info, value) 
		local db = HideParty.db.profile				
		db.targetEnabled = value
		HideParty:UpdateFrames()
		end,
		order = 3,
		},

		raidEnabled = {
		type = "toggle",
		name = L["HideRaid"],
		desc = L["HideRaidDesc"],
		get = function() return db.raidEnabled end,
		set = function(info, value) 
		local db = HideParty.db.profile				
		db.raidEnabled = value
		HideParty:UpdateFrames()
		end,
		order = 4,
		},
	
		playerEnabled = {
		type = "toggle",
		name = L["HidePlayer"],
		desc = L["HidePlayerDesc"],
		get = function() return db.playerEnabled end,
		set = function(info, value) 
		local db = HideParty.db.profile				
		db.playerEnabled = value
		HideParty:UpdateFrames()
		end,
		order = 2,
		},

		partyEnabled = {
		type = "toggle",
		name = L["HideParty"],
		desc = L["HidePartyDesc"],
		get = function() return db.partyEnabled end,
		set = function(info, value) 
		local db = HideParty.db.profile				
		db.partyEnabled = value
		HideParty:UpdateFrames()
		end,
		order = 4,
		},
	},
}

local defaults = {
	profile = {
	raidEnabled = true,
	partyEnabled = true,
	playerEnabled = true,
	targetEnabled = true,
	},
}

function HideParty:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HidePartyDB", defaults, true)
	db = self.db.profile
	LibStub("AceConfig-3.0"):RegisterOptionsTable("HideParty", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HideParty", "HideParty")
	self:RegisterChatCommand("hideparty", "ChatCommand")
	print (L["OnLoad"])
	print (L["OnLoad2"])
end

--Functions--
function HideParty:UpdateFrames()
	--Raid 
	if self:IsEnabled() and db.raidEnabled then
		if not raidenabled then 
			isRaidHidden = true
			HideParty:Raid()
		end
	else
		if isRaidHidden then 
			isRaidHidden = false
			HideParty:SRaid()
		end
	end
	--Party
	if  self:IsEnabled() and  db.partyEnabled then 
		if not partyEnabled then
	        	isPartyHidden = true
			HideParty:Party()
		end
	else
		if not isPartyHidden then
			isPartyHidden = false
			HideParty:SParty()
		end
	end
	--Target
	if self:IsEnabled() and  db.targetEnabled then 
		if not targetEnabled then
			isTargetHidden = true
			HideParty:Target()
		end
	else
		if not isTargetHidden then
			isTargetHidden = false
			HideParty:STarget()
		end
	end
	--Player
	if  self:IsEnabled() and  db.playerEnabled then 
		if not playerEnabled then
			isPlayerHidden = true
			HideParty:Player()
			HideParty:Class()
		end
	else
		if isPlayerHidden then 
			isPlayerHidden = false
			HideParty:SPlayer()
		end
	end
end

function HideParty:OnEnable()
	self:UpdateFrames()
end

function HideParty:OnDisable()
	self:UpdateFrames()
end

function HideParty:OnProfileChanged(db, name)
	self:UpdateFrames()
end

function HideParty:Party()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame:UnregisterAllEvents()
		frame:Hide()
		frame.Show = function() end
	end
	
	UIParent:UnregisterEvent("RAID_ROSTER_UPDATE")

	if not c_400 or c_410 then return end
	CompactPartyFrame:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	CompactPartyFrame:UnregisterEvent("RAID_ROSTER_UPDATE")
	CompactPartyFrame:Hide()
end

function HideParty:SParty()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame.Show = nil
		frame:GetScript("OnLoad")(frame)
		frame:GetScript("OnEvent")(frame, "PARTY_MEMBERS_CHANGED")
		
		PartyMemberFrame_UpdateMember(frame)
	end

	UIParent:RegisterEvent("RAID_ROSTER_UPDATE")

	if not c_400 or c_410 then return end
	CompactPartyFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	CompactPartyFrame:RegisterEvent("RAID_ROSTER_UPDATE")
	if GetDisplayedAllyFrames then
		if GetDisplayedAllyFrames() == "compact-party" then
			CompactPartyFrame:Show()
		end
	elseif GetCVarBool("useCompactPartyFrames") and GetNumPartyMembers() > 0 and GetNumRaidMembers() == 0 then
		CompactPartyFrame:Show()
	end
end

function HideParty:Player()
	PlayerFrame:UnregisterAllEvents()
	PlayerFrameHealthBar:UnregisterAllEvents()
	PlayerFrameManaBar:UnregisterAllEvents()
	PlayerFrame:Hide()
end

function HideParty:SPlayer()
	PlayerFrame:GetScript("OnLoad")(PlayerFrame)
	PlayerFrame:Show()
	PlayerFrame:GetScript("OnEvent")(PlayerFrame, "PLAYER_ENTERING_WORLD")
	PlayerFrame:GetScript("OnEvent")(PlayerFrame, "PARTY_MEMBERS_CHANGED")
	PlayerFrame.animFinished = true
	PlayerFrame.inSeat = true
	PlayerFrame.inSequence = true
	PlayerFrame_UpdateArt(PlayerFrame)
end

function HideParty:Target()
	TargetFrame:UnregisterAllEvents()
	TargetFrame:Hide()

	ComboFrame:UnregisterAllEvents()
end

function HideParty:STarget()
	TargetFrame:GetScript("OnLoad")(TargetFrame)
	
	ComboFrame:GetScript("OnLoad")(ComboFrame)
end

function HideParty:Raid()
	if not c_400 then return end
	CompactRaidFrameManager:UnregisterAllEvents()
	CompactRaidFrameManager:Hide()
	c_r = CompactRaidFrameManager_GetSetting("IsShown")
	if c_r and c_r ~= "0" then 
		CompactRaidFrameManager_SetSetting("IsShown", "0")
	end
end

function HideParty:SRaid()
	if not c_400 then return end
	CompactRaidFrameManager:RegisterEvent("RAID_ROSTER_UPDATE")	
	CompactRaidFrameManager:RegisterEvent("PLAYER_ENTERING_WORLD")
	if GetDisplayedAllyFrames then
		if GetDisplayedAllyFrames() == "raid" then
			CompactRaidFrameManager:Show()
		end
	elseif GetNumRaidMembers() > 0 then
		CompactRaidFrameManager:Show()
	end
	if c_r and c_r ~= "0" then
		CompactRaidFrameManager_SetSetting("IsShown", "1")
	end
end

function HideParty:Class()
	PaladinPowerBar:UnregisterAllEvents()
	PaladinPowerBar:Hide()
	RuneFrame:UnregisterAllEvents()
	RuneFrame:Hide()
	ComboFrame:UnregisterAllEvents()
	ComboFrame:Hide()
	ShardBarFrame:UnregisterAllEvents()
	ShardBarFrame:Hide()
end

function HideParty:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(HideParty, "hideparty", input)
	end
end
